@@ -1,5 +1,5 @@
 /* -*- c-basic-offset: 2 -*- vi:set ts=2 sts=2 sw=2:
- * Copyright (c) 2017 Sean Davis <bluesabre@xfce.org>
+ * Copyright (c) 2017-2020 Sean Davis <bluesabre@xfce.org>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -35,6 +35,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <gio/gdesktopappinfo.h>
 
+#include <exo/exo.h>
 
 
 /* for DBG/TRACE */
@@ -81,11 +82,9 @@ enum {
 static guint signals[LAST_SIGNAL] = { 0 };
 
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-G_DEFINE_TYPE (MprisMenuItem, mpris_menu_item, GTK_TYPE_IMAGE_MENU_ITEM)
+G_DEFINE_TYPE_WITH_PRIVATE (MprisMenuItem, mpris_menu_item, GTK_TYPE_IMAGE_MENU_ITEM)
 G_GNUC_END_IGNORE_DEPRECATIONS
 
-#define GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_MPRIS_MENU_ITEM, MprisMenuItemPrivate))
-
 
 
 /* Static Declarations */
@@ -132,7 +131,7 @@ mpris_menu_item_new_with_player (const gchar *player,
 
   menu_item = MPRIS_MENU_ITEM (g_object_new (TYPE_MPRIS_MENU_ITEM, NULL));
 
-  priv = GET_PRIVATE (menu_item);
+  priv = mpris_menu_item_get_instance_private (menu_item);
 
   priv->player = g_strdup(player);
   if (title != NULL)
@@ -145,7 +144,16 @@ mpris_menu_item_new_with_player (const gchar *player,
 
   gtk_widget_add_events (GTK_WIDGET(menu_item), GDK_SCROLL_MASK|GDK_POINTER_MOTION_MASK|GDK_BUTTON_MOTION_MASK);
 
-  gtk_image_set_from_icon_name (GTK_IMAGE (priv->image), icon_name, GTK_ICON_SIZE_LARGE_TOOLBAR);
+  if (g_file_test (icon_name, G_FILE_TEST_EXISTS) && !g_file_test (icon_name, G_FILE_TEST_IS_DIR)) {
+    GdkPixbuf *buf = exo_gdk_pixbuf_new_from_file_at_max_size (icon_name, 24, 24, TRUE, NULL);
+    if (buf != NULL) {
+      gtk_image_set_from_pixbuf (GTK_IMAGE (priv->image), buf);
+    } else {
+      gtk_image_set_from_icon_name (GTK_IMAGE (priv->image), "audio-player", GTK_ICON_SIZE_LARGE_TOOLBAR);
+    }
+  } else {
+    gtk_image_set_from_icon_name (GTK_IMAGE (priv->image), icon_name, GTK_ICON_SIZE_LARGE_TOOLBAR);
+  }
 
   return GTK_WIDGET(menu_item);
 }
@@ -179,7 +187,7 @@ mpris_menu_item_get_player (MprisMenuItem *item)
 
   g_return_val_if_fail (IS_MPRIS_MENU_ITEM (item), NULL);
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   return priv->player;
 }
@@ -194,7 +202,7 @@ mpris_menu_item_set_title (MprisMenuItem *item,
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   if (title == NULL || *title == '\0')
     gtk_label_set_markup_printf_escaped (GTK_LABEL (priv->title_label), "<b>%s</b>", priv->title);
@@ -212,7 +220,7 @@ mpris_menu_item_set_artist (MprisMenuItem *item,
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   if (artist == NULL || *artist == '\0')
     gtk_label_set_label (GTK_LABEL (priv->artist_label), NULL);
@@ -230,7 +238,7 @@ mpris_menu_item_set_can_go_previous (MprisMenuItem *item,
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   priv->can_go_previous = enabled;
 
@@ -250,7 +258,7 @@ mpris_menu_item_set_can_play (MprisMenuItem *item,
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   priv->can_play = enabled;
 
@@ -270,7 +278,7 @@ mpris_menu_item_set_can_pause (MprisMenuItem *item,
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   priv->can_pause = enabled;
 
@@ -295,7 +303,7 @@ mpris_menu_item_set_can_go_next (MprisMenuItem *item,
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   priv->can_go_next = enabled;
 
@@ -315,7 +323,7 @@ mpris_menu_item_set_can_raise (MprisMenuItem *item,
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   priv->can_raise = can_raise;
 }
@@ -330,7 +338,7 @@ mpris_menu_item_set_can_raise_wnck (MprisMenuItem *item,
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   priv->can_raise_wnck = can_raise;
 }
@@ -345,7 +353,7 @@ mpris_menu_item_set_is_running (MprisMenuItem *item,
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   priv->is_running = running;
 
@@ -381,7 +389,7 @@ mpris_menu_item_set_is_playing (MprisMenuItem *item,
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   priv->is_playing = playing;
 
@@ -413,7 +421,7 @@ mpris_menu_item_set_is_stopped (MprisMenuItem *item,
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   priv->is_stopped = stopped;
 
@@ -456,8 +464,6 @@ mpris_menu_item_class_init (MprisMenuItemClass *item_class)
                                         g_cclosure_marshal_VOID__STRING,
                                         G_TYPE_NONE,
                                         1, G_TYPE_STRING);
-
-  g_type_class_add_private (item_class, sizeof (MprisMenuItemPrivate));
 }
 
 
@@ -467,7 +473,7 @@ mpris_menu_item_init (MprisMenuItem *item)
 {
   MprisMenuItemPrivate *priv;
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   priv->title_label = NULL;
   priv->artist_label = NULL;
@@ -492,7 +498,7 @@ mpris_menu_item_finalize (GObject *object)
   MprisMenuItemPrivate *priv;
 
   item = MPRIS_MENU_ITEM (object);
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   if (priv->player)
     g_free (priv->player);
@@ -523,7 +529,7 @@ mpris_menu_item_raise (MprisMenuItem *item)
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   if (priv->is_running)
     {
@@ -550,7 +556,7 @@ mpris_menu_item_launch (MprisMenuItem *item)
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   if (priv->is_running)
     return;
@@ -572,7 +578,7 @@ mpris_menu_item_raise_or_launch (MprisMenuItem *item)
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   if (priv->is_running)
     mpris_menu_item_raise (item);
@@ -592,7 +598,7 @@ mpris_menu_item_get_widget_at_event (MprisMenuItem  *item,
 
   g_return_val_if_fail (IS_MPRIS_MENU_ITEM (item), NULL);
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   gtk_widget_get_allocation (priv->button_box, &alloc);
   gtk_widget_translate_coordinates (GTK_WIDGET (item), priv->button_box, event->x, event->y, &x, &y);
@@ -758,7 +764,7 @@ update_packing (MprisMenuItem *item)
 
   g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
 
-  priv = GET_PRIVATE (item);
+  priv = mpris_menu_item_get_instance_private (item);
 
   TRACE("entering");
 
