@@ -1,4 +1,4 @@
-/*  Copyright (c) 2015 Andrzej <ndrwrdck@gmail.com>
+/*  Copyright (c) 2015-2017 Andrzej <andrzejr@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -23,6 +23,7 @@
 
 #include "pulseaudio-volume.h"
 #include "pulseaudio-config.h"
+#include "pulseaudio-mpris.h"
 
 G_BEGIN_DECLS
 
@@ -40,6 +41,7 @@ typedef struct          _PulseaudioMenuClass         PulseaudioMenuClass;
 
 PulseaudioMenu         *pulseaudio_menu_new         (PulseaudioVolume *volume,
                                                      PulseaudioConfig *config,
+                                                     PulseaudioMpris  *mpris,
                                                      GtkWidget        *widget);
 
 G_END_DECLS
