@@ -1,4 +1,4 @@
-/*  Copyright (c) 2017 Sean Davis <bluesabre@xfce.org>
+/*  Copyright (c) 2017-2020 Sean Davis <bluesabre@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -53,12 +53,14 @@ struct _PulseaudioMprisPlayer
 
   gchar            *title;
   gchar            *artist;
+  gchar            *full_path;
 
   gboolean          can_go_next;
   gboolean          can_go_previous;
   gboolean          can_pause;
   gboolean          can_play;
   gboolean          can_raise;
+  gboolean          can_launch;
 
   PlaybackStatus    playback_status;
 
@@ -139,6 +141,67 @@ pulseaudio_mpris_player_class_init (PulseaudioMprisPlayerClass *klass)
 
 
 
+static gboolean
+pulseaudio_mpris_player_can_condition_track_info (PulseaudioMprisPlayer *player)
+{
+  if (player->title == NULL || player->artist == NULL)
+    return FALSE;
+
+  if (!g_utf8_validate (player->title, -1, NULL) || !g_utf8_validate (player->artist, -1, NULL))
+    return FALSE;
+
+  if (g_utf8_strlen (player->title, -1) <= 0 || g_utf8_strlen (player->artist, -1) <= 0)
+    return FALSE;
+
+  return TRUE;
+}
+
+
+static gboolean
+pulseaudio_mpris_player_condition_track_info (PulseaudioMprisPlayer *player,
+                                              const gchar           *delimiter)
+{
+  gchar    *lookup = NULL;
+  gchar    *replace = NULL;
+  gboolean  found = FALSE;
+
+  lookup = g_strconcat (player->artist, delimiter, NULL);
+  if (g_str_has_prefix (player->title, lookup))
+    {
+      replace = g_utf8_substring (player->title, g_utf8_strlen (lookup, -1), g_utf8_strlen (player->title, -1));
+      g_free (player->title);
+      player->title = replace;
+      found = TRUE;
+    }
+  g_free (lookup);
+
+  if (found)
+    return TRUE;
+
+  // Track titles match ARTIST - TITLE
+  if (g_str_has_suffix (player->artist, "VEVO"))
+    {
+      gchar **components = g_strsplit (player->title, delimiter, 2);
+
+      if (g_strv_length (components) == 2)
+        {
+          g_free (player->artist);
+          player->artist = g_strdup (components[0]);
+
+          g_free (player->title);
+          player->title = g_strdup (components[1]);
+
+          found = TRUE;
+        }
+
+      g_strfreev (components);
+    }
+
+  return found;
+}
+
+
+
 static void
 pulseaudio_mpris_player_parse_metadata (PulseaudioMprisPlayer *player,
                                         GVariant              *dictionary)
@@ -170,22 +233,38 @@ pulseaudio_mpris_player_parse_metadata (PulseaudioMprisPlayer *player,
         }
       else if (0 == g_ascii_strcasecmp (key, "xesam:artist"))
         {
-          artists = g_variant_dup_strv (value, NULL);
-          if (artists != NULL)
+          if (g_variant_is_of_type (value, G_VARIANT_TYPE_STRING))
             {
-              if (g_strv_length (artists) > 0)
+              player->artist = g_strdup (g_variant_get_string(value, NULL));
+            }
+          else
+            {
+              artists = g_variant_dup_strv (value, NULL);
+              if (artists != NULL)
                 {
-                  player->artist = g_strdup (artists[0]);
-                }
-              else
-                {
-                  player->artist = g_strdup ("");
-                }
+                  if (g_strv_length (artists) > 0)
+                    {
+                      player->artist = g_strdup (artists[0]);
+                    }
+                  else
+                    {
+                      player->artist = g_strdup ("");
+                    }
 
-              g_strfreev (artists);
+                  g_strfreev (artists);
+                }
             }
         }
     }
+
+  if (!pulseaudio_mpris_player_can_condition_track_info (player))
+    return;
+
+  if (pulseaudio_mpris_player_condition_track_info (player, " - "))
+    return;
+
+  if (pulseaudio_mpris_player_condition_track_info (player, ": "))
+    return;
 }
 
 
@@ -424,6 +503,12 @@ pulseaudio_mpris_player_parse_playback_status (PulseaudioMprisPlayer *player,
 
 
 
+static void
+pulseaudio_mpris_player_set_details_from_desktop (PulseaudioMprisPlayer *player,
+                                                  const gchar           *player_name);
+
+
+
 static void
 pulseaudio_mpris_player_parse_player_properties (PulseaudioMprisPlayer *player,
                                                  GVariant              *properties)
@@ -490,6 +575,7 @@ pulseaudio_mpris_player_parse_media_player_properties (PulseaudioMprisPlayer *pl
   GVariantIter  iter;
   GVariant     *value;
   const gchar  *key;
+  const gchar  *filename = NULL;
 
   g_variant_iter_init (&iter, properties);
 
@@ -499,6 +585,11 @@ pulseaudio_mpris_player_parse_media_player_properties (PulseaudioMprisPlayer *pl
         {
           player->can_raise = g_variant_get_boolean(value);
         }
+      else if (0 == g_ascii_strcasecmp (key, "DesktopEntry"))
+        {
+          filename = g_variant_get_string(value, NULL);
+          pulseaudio_mpris_player_set_details_from_desktop (player, filename);
+        }
     }
 }
 
@@ -641,14 +732,13 @@ static gchar *
 find_desktop_entry (const gchar *player_name)
 {
   GKeyFile  *key_file;
-  gchar     *file;
+  gchar     *file = NULL;
   gchar     *filename = NULL;
-  gchar     *full_path;
 
   file = g_strconcat ("applications/", player_name, ".desktop", NULL);
 
   key_file = g_key_file_new();
-  if (g_key_file_load_from_data_dirs (key_file, file, &full_path, G_KEY_FILE_NONE, NULL))
+  if (g_key_file_load_from_data_dirs (key_file, file, NULL, G_KEY_FILE_NONE, NULL))
     {
       filename = g_strconcat (player_name, ".desktop", NULL);
     }
@@ -673,7 +763,9 @@ find_desktop_entry (const gchar *player_name)
     }
 
   g_key_file_free (key_file);
-  g_free (file);
+
+  if (file)
+    g_free (file);
 
   return filename;
 }
@@ -724,8 +816,10 @@ pulseaudio_mpris_player_set_details_from_desktop (PulseaudioMprisPlayer *player,
       player->icon_name = g_strdup ("applications-multimedia");
     }
 
-  if (full_path != NULL)
+  if (full_path != NULL) {
+    player->full_path = g_strdup (full_path);
     g_free (full_path);
+  }
 
   g_key_file_free (key_file);
   g_free (file);
@@ -771,6 +865,8 @@ pulseaudio_mpris_player_set_player (PulseaudioMprisPlayer *player,
 
   pulseaudio_mpris_player_set_details_from_desktop (player, player_name);
   pulseaudio_mpris_player_dbus_connect (player);
+
+  player->can_launch = player->full_path != NULL;
 }
 
 
@@ -905,6 +1001,14 @@ pulseaudio_mpris_player_get_artist (PulseaudioMprisPlayer *player)
 
 
 
+const gchar *
+pulseaudio_mpris_player_get_full_path (PulseaudioMprisPlayer *player)
+{
+  return player->full_path;
+}
+
+
+
 gboolean
 pulseaudio_mpris_player_is_connected (PulseaudioMprisPlayer *player)
 {
@@ -969,6 +1073,14 @@ pulseaudio_mpris_player_can_raise (PulseaudioMprisPlayer *player)
 
 
 
+gboolean
+pulseaudio_mpris_player_can_launch (PulseaudioMprisPlayer *player)
+{
+  return player->can_launch;
+}
+
+
+
 gboolean
 pulseaudio_mpris_player_is_equal (PulseaudioMprisPlayer *a,
                                   PulseaudioMprisPlayer *b)
@@ -990,6 +1102,7 @@ pulseaudio_mpris_player_init (PulseaudioMprisPlayer *player)
 
   player->title             = NULL;
   player->artist            = NULL;
+  player->full_path         = NULL;
 
   player->can_go_next       = FALSE;
   player->can_go_previous   = FALSE;
@@ -1020,6 +1133,7 @@ pulseaudio_mpris_player_finalize (GObject *object)
 
   player->title             = NULL;
   player->artist            = NULL;
+  player->full_path         = NULL;
 
   player->can_go_next       = FALSE;
   player->can_go_previous   = FALSE;
